#![no_std]
#![no_main]
#![feature(async_fn_track_caller, type_alias_impl_trait)]
#![warn(rust_2018_idioms)]
#![forbid(elided_lifetimes_in_paths, unsafe_code)]

mod static_mutex;

use defmt::*;
use defmt_rtt as _;
use embassy_executor::Spawner;
use embassy_stm32::{
	exti::ExtiInput,
	gpio::{Input, Level, Output, Pull, Speed},
	peripherals::{PA5, PC13}
};
use embassy_sync::blocking_mutex::raw::ThreadModeRawMutex;
use embassy_time::{Duration, Timer};
use panic_probe as _;
use static_mutex::StaticMutex;

type Mutex<T> = embassy_sync::mutex::Mutex<ThreadModeRawMutex, T>;
type MutexGuard<'a, T> = embassy_sync::mutex::MutexGuard<'a, ThreadModeRawMutex, T>;

type NucleoLed = Output<'static, PA5>;
type NucleoButton = ExtiInput<'static, PC13>;

static LED: StaticMutex<NucleoLed> = StaticMutex::new();
static BLINKING: Mutex<bool> = Mutex::new(true);

#[embassy_executor::task]
async fn blinker(interval: Duration) {
	loop {
		if *BLINKING.lock().await {
			LED.lock().await.toggle();
		}
		Timer::after(interval).await;
	}
}

#[embassy_executor::task]
async fn button_handler(mut button: NucleoButton) {
	loop {
		button.wait_for_falling_edge().await;
		let mut blinking = BLINKING.lock().await;
		if *blinking {
			info!("Disable blinking");
			LED.lock().await.set_low();
			*blinking = false;
		} else {
			info!("Enable blinking");
			*blinking = true;
		}
	}
}

#[embassy_executor::main]
async fn main(spawner: Spawner) {
	let p = embassy_stm32::init(Default::default());

	LED.init(Output::new(p.PA5, Level::Low, Speed::Low)).await;
	let button = ExtiInput::new(Input::new(p.PC13, Pull::Up), p.EXTI13);

	unwrap!(spawner.spawn(blinker(Duration::from_millis(500))));
	unwrap!(spawner.spawn(button_handler(button)));
}
